      PROGRAM RNJ46A
C
C     ******************************************************************
C     *                                                                *
C     *           DETERMINATION OF OPTICAL CONSTANTS FROM ABSORBANCE   *
C     *                                                                *
C     *                    MEASUREMENTS ON THIN LIQUID FILMS.          *
C     *                                                                *
C     *                            VERSION 08/92                       *
C     *                                                                *
C     ******************************************************************
C
C     BASED ON THE PROGRAM WRITTEN BY:
C     D.G.CAMERON,J.P.HAWRANEK,P.NEELAKANTAN,R.P.YOUNG AND R.N.JONES,
C       DIVISION OF CHEMISTRY,NATIONAL RESEARCH COUNCIL OF CANADA,
C       OTTAWA,CANADA.
C
C     MODIFIED BY J. E. BERTIE AND C. D. KEEFE, UNIVERSITY OF ALBERTA, CANADA
C     TO RUN ON THE IBM COMBATIBLE PC COMPUTER.
C     SPECTRAL INPUT IS EITHER A SPECTRACALC ABSORBANCE FILE OR AN ASCII FILE
C     IN WHICH THE DATA POINTS ARE XSTART, XEND, NUMBER OF POINTS, ABSORANCE
C     DATA
C
C     MODIFIED ON 18/3/91 TO ALLOW FOR THE INPUT ABSORBANCE SPECTRUM TO BE
C     BASELINE CORRECTED USING THE ABSORBANCE PER MICRON GIVEN BY THE USER
C     AT TWO DIFFERENT WAVENUMBERS.  FOR DETAILS OF BASELINE CORRECTION
C     PROCEDURE SEE J.E. BERTIE, C.D. KEEFE AND R.N. JONES, CAN. J. CHEM.
C     69, 1609 (1991)  
C
C     MODIFIED IN VERSION 06/92 TO ALLOW THE BASELINE CORRECTION TO BE  
C     PERFORMED AT MORE THAN TWO POINTS.  
C
C     MODIFIED IN VERSION 07/92 TO HAVE THE BASELINE CORRECTION PREFORMED 
C     DURING EACH ITERATION. ALSO THE TABUALTION OF N, KAPP, K,
C     E' AND E'' WAS REMOVED. 
C
C     MODIFIED IN VERSION 08/92 TO ALLOW THE INPUT OF AN N SPECTRUM   
C     TO CALCULATE THE K SPECTRUM
C
C     THE COMPLEX FRESNEL COEFFICENTS WERE CONVERTED TO REAL IN VERSION 04/91.
C     THIS MADE THE WAVENUMBER LOOP MUCH FASTER.  INORDER TO DO THIS THE
C     CACLUATION OF KAPP HAD TO BE DONE WITH A DIFFERENT ROUTINE THAN THE
C     OTHER K'S SINCE WHEN THE LIQUID IS ASSUMED TO BE NONABSORBING, THE
C     REFRACTIVE INDEX OF THE LIQUID IS REAL (I.E. IMAGINARY IS ZERO) WHICH
C     CAUSED A DIVISION BY ZERO ERROR.
C
C     THIS PROGRAM CALCULATES OPTICAL CONSTANTS FROM THE EXPERIMENTALLY
C     MEASURED INFRARED ABSORBANCE SPECTRUM OF A THIN ABSORBING
C     FILM BETWEEN WINDOWS OF GIVEN REFRACTIVE INDICIES (WHICH MAY BE 1).
C     IT CALCULATES THE SAMPLE'S REAL REFRACTIVE INDICES FROM ITS APPARENT
C     IMAGINARY REFRACTIVE INDICES VIA A KRAMERS-KRONIG TRANSFORMATION.
C     IT ITERATES TO CONSTANT VALUES OF REAL AND IMAGINARY REFRACTIVE INDICES.
C     IT OUTPUTS OPTICAL CONSTANTS.
C
C     FOR DETAILED DISCUSSION SEE R.N.JONES ET AL SPECTROCHIMICA ACTA
C       VOL 32A.PP.75,85,99,111 (1976).
C
C     THE ITERATION IS TERMINATED BY THE FOLLOWING CONDITIONS:
C       A. THE PROCESS BECOMES DIVERGENT.
C       B. THE MEAN VALUE OF THE CORRECTIONS (DEV) IS LESS THAN 0.00002
C       C. THE MAXIMUM NUMBER OF PERMITTED ITERATIONS HAS BEEN
C           REACHED (ITMAX).
C       FOR B AND C THE LAST SET OF CORRECTIONS ARE APPLIED TO THE K
C           VECTOR AND THE SPECTRUM IS EVALUATED.
C       DCAY IS THE VECTOR OF THE CORRECTIONS TO K ON THE LAST
C           ITERATION CYCLE.
C       T IS THE CORRECTED TRANSMISSION SPECTRUM CALCULATED
C           FROM THE FINAL K VALUES.
C
C     FEATURES: 
C             INTEGRATION OVER ANGLE AND PATH LENGTH RANGES USING
C                A 5/5 DOUBLE GAUSSIAN QUADRATURE.
C             CALCULATION WITH OR WITHOUT SPECTROMETER POLARIZATION
C                DISCRIMINATION.
C
C      MODE:   0 NO KK TRANSFORM, N IS READ FROM A FILE AND HELD CONSTANT
C              1 USE SUBTRACTIVE KK
C              2 USE STANDARD KK               
C
C       NP:   NUMBER OF DATA POINTS
C
C      DELX:   WAVENUMBER SPACING
C
C      XSP:   WAVENUMBER START POINT
C
C      XEP:   WAVENUMBER END POINT
C
C     ITMAX:   THE MAXIMUM NUMBER OF PERMITTED ITERATIONS
C             (RECOMMENDED VALUE IS 5, IF ONLY KAPP AND NAPP ARE
C             REQUIRED USE ITMAX=0)
C
C     ICELL: IDENTIFIES THE COMMONLY USED WINDOW MATERIALS
C           1 GERMANIUM
C           2 LITHIUM FLUORIDE
C           3 BARIUM FLUORIDE
C           4 CALCIUM FLUORIDE
C           5 CESIUM IODIDE
C           6 POTASSIUM BROMIDE
C           7 SODIUM CHLORIDE
C           0 OTHER    NRI DATA POINTS OF WAVENUMBER-REFRACTIVE
C             MUST THEN BE ENTERED IN ARRAY RNU,RIN.
C
C      NPC: NUMBER OF WAVENUMBER-SPECTROMETER POLARIZATION
C           DISCRIMINATION (PNU,PC) DATA POINTS. USE NPC=0 FOR NO
C           POLARIZATION DISCRIMINATION.
C
C     BREAK1,BREAK2: POSITIONS OF WAVENUMBER DISCONTINUITIES IN THE
C                  POLARIZATION DISCRIMINATION FUNCTION.AVOID PNU VALUES
C                  EQUAL TO BREAK1 OR BREAK2.
C
C      FYI: MAXIMUM CONVERGENCE ANGLE IN DEGREES.
C
C     DM0,DM1: THE MEAN PATH LENGTH AND WEDGE PARAMETER OF THE CELL
C             IN MICROMETRES.
C
C      ODY: ABSORBANCE DATA POINTS. NOTE THAT THE
C           CONSTANT WAVENUMBER INTERVAL (DELX) MUST BE MAINTAINED
C           THROUGHOUT THE SPECTRUM.
C
C
C      IWRTA: IF IWRTA=0 THE BASELINE CORRECTION PROCEDURE IS IGNORED
C             IF IWRTA >0 CORRECTED ABSORBANCE IS WRITTEN TO DISK AND
C             IWRTA REFERENCE POINTS MUST BE INPUT.
C             IF IWRTA <0 CORRECTED ABSORBANCE IS NOT WRITTEN TO DISK AND
C             ABS(IWRTA) REFERENCE POINTS MUST BE INPUT.
C
C      WN: WAVENUMBER VALUES USED FOR BASELINE CORRECTION
C
C     AREF: ABSORBANCE / CM (I.E. 10**4 ABS/MICRON) AT THE VALUES OF WN
C
C     THE REQUIRED INPUTS ARE(IN RNJ46A.ASC):
C      THE NUMBER OF RUNS
C      FOR EACH RUN THE FOLLOWING INFO IS NEEDED
C      - 1 COMMENT LINE THAT RNJ46A.EXE WILL IGNORE
C      - INPUT FILE NAME (IN SINGLE QUOTATION MARKS)
C      - TWO COMMENT LINE THAT WILL BE PRINTED IN THE OUTPUT FILE
C      - MODE, ITMAX, ICELL, NRI, NPC, FYI, DM0, DM1
C      - 'NFILE'  (IF MODE=0)
C      - ENR      (IF MODE=1)
C      - WR, ENR  (IF MODE=2)
C      - IWRTA
C      - ABS(IWRTA) DATA POINTS OF WN AND AREF
C      - NRI DATA POINTS OF NU AND REFRACIVE INDEX (ONLY IF ICELL=0),
C      - NPC WAVENUMBER-SPECTROMETER POLARIZATION DISCRIMINATION (PNU,PC)
C       DATA POINTS (ONLY IF NPC>0),
C      - BREAK1, BREAK2 (BREAK POINTS ONLY NEEDED IF NPC>0).
C      NOTE THAT - INDICATES START OF NEW LINE, BUT APART FROM THIS
C      THE PARAMETERS CAN BE SEPERATED BY A SPACE, COMMA, OR CARRIAGE RETURN.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INCLUDE 'RNJ46A.PAR'
      PARAMETER (IRD=5,IPRNT=6, ND=5, NFY=5, NWNMAX=50)
      DIMENSION HEAD1(20),HEAD2(20),ODY(NPMAX),PNU(90),PC(90),RNU(40),
     $ RI(NPMAX),POLS(NPMAX),RIN(40),RA(2),
     $ TA(2),TC(2),TT(2),CX(2),CXR1(2),
     $ CXR2(2),CXT1(2),CXT2(2),CXR1SQ(2),CXR2SQ(2),CXR12(2),CXR12S(2),
     $ CXT12S(2),TATC(2),
     $ RCXSQ(2),ANGCX(2),RCXR1(2),RCXR1S(2),ANCXR1(2),
     $ RCXT1S(2),RCXT2S(2),RCXR2(2),ANCXR2(2), WN(NWNMAX), AREF(NWNMAX)
      REAL*8 AG(5),XG(5),TP,DBLX,RIT2,XS,XG1(5)
      CHARACTER*12 INFILE,OUTFILE,KFILE,NFILE,CAFILE
      CHARACTER*4 CEXT
      CHARACTER*192 COMM
      INTEGER*1 XT,YT
      LOGICAL DOSPC, NFIX
      COMMON/CKK/ X(NPMAX),XS(NPMAX),CAY(NPMAX),EN(NPMAX),ENR,WR,DELX,
     $ NP,MODE
C
C     COEFFS AG AND NODES XG FOR THE GAUSSIAN QUADRATURE
C
      DATA AG, XG, XG1 /0.236926885056189D0,0.478628670499366D0,
     $0.568888888888889D0,0.478628670499366D0,0.236926885056189D0,
     $0.906179845938664D0,0.538469310105683D0,0.0D0,
     $-0.538469310105683D0,-0.906179845938664D0,
     $1.906179845938664D0,1.538469310105683D0,1.0D0,
     $0.461530689894317D0,0.093820154061336D0/
C
      WRITE(*,1150)
      IRUN=0
      OPEN (IRD,FILE='RNJ46A.ASC',STATUS='OLD',MODE='READ')
      READ (IRD,*)IRUNT
   1  PRINT *, 'READING DATA FROM RNJ46A.ASC FOR RUN ',IRUN+1
      CALL GETTIM(IH,IM,IS,IS100)
      WRITE(*,1140) 'TIME BEFORE RUN IS',IM,IS,IS100
      RES=0.0
      IRUN=IRUN+1
      READ (IRD,1020)HEAD1
      READ (IRD,*)INFILE
      DOSPC=.FALSE.
      IPOS=9
      DO 5 I=9,2,-1
	IF(INFILE(I:I).EQ.'.')THEN
	  IPOS=I
	  GO TO 6
	END IF
   5  CONTINUE
   6  CEXT=INFILE(IPOS:IPOS+3)
      IF(CEXT.EQ.'.SPC'.OR.CEXT.EQ.'.spc')DOSPC=.TRUE.
      OUTFILE=INFILE(:IPOS-1)//'.OUT'
      KFILE=INFILE(:IPOS-2)//'K'//CEXT
      NFILE=INFILE(:IPOS-2)//'N'//CEXT
      CAFILE=INFILE(:IPOS-2)//'C'//CEXT
      OPEN (IPRNT,FILE=OUTFILE,STATUS='UNKNOWN',MODE='WRITE')
      WRITE (IPRNT,1010)
      READ (IRD,1020) HEAD1
      READ (IRD,1020) HEAD2
      READ (IRD,*) MODE,ITMAX,ICELL,NRI,NPC,FY1,DM0,DM1
      NFIX=.FALSE.
      IF (MODE.EQ.0)THEN
	READ(IRD,*)NFILE
	NFIX=.TRUE.
	IF(DOSPC)THEN
	  CALL READSC(NFILE,EN,NP,XSPN,XEPN,XT,YT,RES,COMM,IER)
	ELSE
	  CALL READAS(NFILE,EN,NP,XSPN,XEPN)
	END IF
	DELXN=ABS((XSPN-XEPN)/(NP-1))
	WRITE(IPRNT,1190)NFILE
	IF(NP.GT.NPMAX)THEN
	  WRITE(*,*)'INPUT N FILE TOO LARGE. MAX. NUMBER OF POINTS ',
     $     NPMAX
	  WRITE(IPRNT,*)'INPUT N FILE TOO LARGE. MAX. NUMBER OF POINTS '
     $    ,NPMAX
	  GO TO 120
      END IF
      ELSE IF (MODE.EQ.1)THEN
	READ (IRD,*) WR,ENR
	WRITE (IPRNT,1080) WR,ENR
      ELSE
	READ (IRD,*) ENR
	WRITE (IPRNT,1090) ENR
      END IF
      EN2=ENR
      READ(IRD,*) IWRTA
      IF(IWRTA.NE.0)READ(IRD,*)(WN(I),AREF(I),I=1,ABS(IWRTA))
      IF (ICELL.EQ.0) READ (IRD,*) (RNU(I),RIN(I),I=1,NRI)
      IF (NPC.NE.0)THEN
	READ (IRD,*) (PNU(I),PC(I),I=1,NPC)
	READ (IRD,*) BREAK1,BREAK2
      END IF
      IF(DOSPC)THEN
	CALL READSC(INFILE,ODY,NP,XSP,XEP,XT,YT,RES,COMM,IER)
      ELSE
	CALL READAS(INFILE,ODY,NP,XSP,XEP)
      END IF
      IF(NP.GT.NPMAX)THEN
	WRITE(*,*)'INPUT FILE TOO LARGE. MAX. NUMBER OF POINTS ',NPMAX
	WRITE(IPRNT,*)'INPUT FILE TOO LARGE. MAX. NUMBER OF POINTS ',
     $ NPMAX
	GO TO 120
      END IF
      IF(ABS(IWRTA).EQ.1)THEN
	 WRITE(IPRNT,*)'Baseline correction requested but to few', 
     $   ' reference points supplied.'
	 GO TO 120
      END IF
      IF(IWRTA.NE.0.AND.(WN(1).GT.XSP.OR.WN(ABS(IWRTA)).LT.XEP))THEN
	WRITE(IPRNT,*)'  Wavenumbers for baseline correction, out of', 
     $  ' range of file.'
	GO TO 120
      END IF
      DELX=ABS((XSP-XEP)/(NP-1))
      WRITE (IPRNT,1130) HEAD1
      WRITE (IPRNT,1130) HEAD2
      IF(NFIX)THEN
	IF(ABS(DELXN-DELX).GT.0.01)THEN
	  WRITE(*,1200)
	  WRITE(IPRNT,1200)
	  GO TO 120
	END IF
	IF((XSP-XSPN).GT.0.01.OR.(XEPN-XEP).GT.0.01)THEN
	  WRITE(*,1210)
	  WRITE(IPRNT,1210)
	  GO TO 120
	ELSE IF((XSPN-XSP).GT.0.01)THEN
	  IN1=(XSPN-XSP)/DELX
	  DO 7 I = 1,NP
	    EN(I)=EN(IN1+1)
 7        CONTINUE
	END IF
      END IF
      IF(IWRTA.EQ.0)THEN
	WRITE (IPRNT,1135)
      ELSE
	WRITE (IPRNT,1136)
      END IF
      DO 10 I=1,7
	TP=7.71386817D0/2**I
	AUX=ABS(DELX-TP)
	IF(AUX.LE.0.01)GO TO 15
 10   CONTINUE
      WRITE(IPRNT,1000)DELX
      GO TO 16
 15   DELX=TP
 16   IF(NFIX)THEN
	WRITE (IPRNT,1030) EN2,FY1,DM0,DM1,ICELL
      ELSE
	WRITE(IPRNT,1220)FY1,DM0,DM1,ICELL
      END IF
C
      DM0=0.0001*DM0
      DM1=0.0001*DM1
      FPDM0=5.457505*DM0
      FY1=0.00872665*FY1
C
      DO 25 I=1,NP
	X(I)=XSP-DELX*(I-1)
	XS(I)=X(I)*X(I)
   25 CONTINUE
      IF (ICELL.EQ.0)THEN
	CALL RIINT2 (X,RNU,RIN,RI,NP,NRI)
      ELSE
	DO 35 I=1,NP
	  DBLX=DBLE(X(I))
	  RI(I)=RIT2(DBLX,ICELL)
   35  CONTINUE
      END IF
      CALL POLCO2 (X,PNU,PC,NPC,NP,BREAK1,BREAK2,POLS)
C
C     START OF THE ITERATION
      CALL GETTIM(IH,IM,IS,IS100)
      WRITE(*,1140) 'START OF ITERATIONS',IM,IS,IS100
C
C     THE BASELINE AND K(A) AND N(A) SPECTRA ARE CALCULATED FIRST
      WRITE(*,*)'CALCULATION OF KAPP'
      DO 45 IT=0,ITMAX
	PRINT *,'ITERATION #',IT
	DEV=0.0
C
C      START OF THE WAVENUMBER LOOP
C
	CALL GETTIM(IH,IM,IS,IS100)
	WRITE(*,1140) 'START OF WAVENUMBER LOOP',IM,IS,IS100
	DO 80 I=1,NP
	  W=X(I)
	  FRPIW=12.5663706*W
	  EN1=RI(I)
	  EN1SQ=EN1*EN1
	  T=0.0
C
C        START OF THE ANGLE LOOP
C
	  DO 70 IFI=1,NFY
	    FI0=FY1*XG1(IFI)
	    CF0=COS(FI0)
	    SF0=SIN(FI0)
	    CF1SQ=1.0-SF0*SF0/EN1SQ
	    CF1=SQRT(CF1SQ)
C
C          R AND T AT A AND C BOUNDARIES
C
	    ENC1=EN1*CF1
	    ENC1SQ=ENC1*ENC1
	    B=CF0+ENC1
	    B=1/(B*B)
	    A1=4.0*CF0*CF0
	    TA(1)=A1*B
	    A2=4.0*ENC1*ENC1
	    TC(1)=A2*B
	    A3=CF0-ENC1
	    RA(1)=A3*A3*B
	    B=CF1+EN1*CF0
	    B=1/(B*B)
	    TA(2)=A1*B
	    TC(2)=A2*B
	    A3=EN1*CF0-CF1
	    RA(2)=A3*A3*B
C
C          START OF BASELINE-SAMPLE LOOP
C          CALCULATION OF THE FRESNEL COEFFICIENTS
C
	    IF (IT.EQ.0.OR.CAY(I).EQ.0)THEN
	      IF(IT.GT.0.OR.NFIX)EN2=EN(I)
	      CXA=SF0/EN2
	      CXCOS=SQRT(1.0-CXA*CXA)
	      CXENC=EN2*CXCOS
C            1 DENOTES THE S COMPONENT, 2 THE P COMPONENT
	      CX(1)= ENC1+CXENC
	      CX(2)= EN1*CXCOS+EN2*CF1
	      CXR1(1)=(ENC1-CXENC)/CX(1)
	      CXR1(2)=(EN2*CF1-EN1*CXCOS)/CX(2)
	      DO 550 IP=1,2
		CXT1(IP)=2.0*ENC1/CX(IP)
		CXT2(IP)=2.0*CXENC/CX(IP)
		CXR2(IP)=-CXR1(IP)
		CXR1SQ(IP)=CXR1(IP)*CXR1(IP)
		CXR2SQ(IP)=CXR1SQ(IP)
		CXR12(IP)=-CXR2SQ(IP)
		CXR12S(IP)=CXR1SQ(IP)*CXR2SQ(IP)
		CXT12S(IP)=CXT1(IP)*CXT1(IP)*CXT2(IP)*CXT2(IP)
		TATC(IP)=TA(IP)*TC(IP)
  550        CONTINUE
	      TD=0.0
	      CXENC=CXENC*W*6.283148
C
C           START OF THE PATH LENGTH LOOP
C
	      DO 650 ID=1,ND
		D=DM0+DM1*XG(ID)
		CXA=D*CXENC
		CA=COS(CXA)
		CA2=COS(2.*CXA)
C
C             R AND T AT B BOUNDARY ( THIN FILM )
		DO 600 IP=1,2
		  CXA=CXR1SQ(IP)+2.*CXR12(IP)*CA2+CXR2SQ(IP)
		  CXB=1./(1.+2.*CXR12(IP)*CA2+CXR12S(IP))
		  RB=CXA*CXB
		  TB=CXT12S(IP)*CXB
C
C               TOTAL T
C
		  RAIP=RA(IP)
		  W1=RAIP*RB
		  W2=1.0/(1.0-W1)
		  RBA=(RAIP+RB-2.0D0*W1)*W2
		  W1=RBA*RAIP
		  TT(IP)=TATC(IP)*TB*W2/(1.0-W1)
  600         CONTINUE
		TD=TD+AG(ID)*(POLS(I)*(TT(1)-TT(2))+TT(2))
  650        CONTINUE
	    ELSE
	      RN=EN(I)
	      RK=CAY(I)
	      RNKSQ=RN*RN+RK*RK
	      RNK=SQRT(RNKSQ)
	      ANGNK=ATAN(RK/RN)
C
	      CXA=SF0/RNK
	      CXA=CXA*CXA
	      ANGCXA=-2.*ANGNK
	      CXA2SN=CXA*SIN(ANGCXA)
	      CXA2CS=CXA*COS(ANGCXA)
C
	      RCXCS2=SQRT(1.-2.*CXA2CS+CXA*CXA)
	      RCXCOS=SQRT(RCXCS2)
	      ACXCOS=0.5*ATAN(-CXA2SN/(1.-CXA2CS))
	      RCOS3=RCXCOS*COS(ACXCOS)
	      RSIN3=RCXCOS*SIN(ACXCOS)
C
	      RNKCOS=RNK*RCXCOS
	      RNKCS2=RNKCOS*RNKCOS
	      ACSPNK=ACXCOS+ANGNK
	      ACSMNK=ACXCOS-ANGNK
	      RCOS1=RNKCOS*COS(ACSPNK)
	      RCOS2=RNKCOS*COS(ACSMNK)
	      RSIN1=RNKCOS*SIN(ACSPNK)
	      RSIN2=RNKCOS*SIN(ACSMNK)
C
C            1 DENOTES THE S COMPONENT, 2 THE P COMPONENT
	      RCXSQ(1)=ENC1SQ+2.*ENC1*RCOS1+RNKCS2
	      ANGCX(1)=ATAN(RSIN1/(ENC1+RCOS1))
	      RCXSQ(2)=EN1SQ*RCXCS2+CF1SQ*RNKSQ+2.*RCOS2*ENC1
	      ANGCX(2)=ATAN((RSIN3*EN1+CF1*RK)/(RCOS3*EN1+CF1*RN))
	      RCXR1S(1)=(ENC1SQ-2.*RCOS1*ENC1+RNKCS2)/RCXSQ(1)
	      RCXR1(1)=SQRT(RCXR1S(1))
	      ANCXR1(1)=ATAN(-RSIN1/(ENC1-RCOS1))-ANGCX(1)
	      RCXR1S(2)=(CF1SQ*RNKSQ+EN1SQ*RCXCS2-2.*ENC1*RCOS2)/
     $        RCXSQ(2)
	      RCXR1(2)=SQRT(RCXR1S(2))
	      ANCXR1(2)=ATAN((CF1*RK-EN1*RSIN3)/(CF1*RN-EN1*
     $        RCOS3))-ANGCX(2)
	      DO 55 IP=1,2
		RCXT1S(IP)=4.*ENC1SQ/RCXSQ(IP)
		RCXT2S(IP)=4.*RNKCS2/RCXSQ(IP)
		RCXR2(IP)=-RCXR1(IP)
		ANCXR2(IP)=ANCXR1(IP)
		TATC(IP)=TA(IP)*TC(IP)
55           CONTINUE
	      TD=0.0
C
C            START OF THE PATH LENGTH LOOP
C
	      DO 65 ID=1,ND
		D=DM0+DM1*XG(ID)
C             CXSA IS I*DELTA
C             R AND T AT B BOUNDARY ( THIN FILM )
		RCXE2=EXP(-D*FRPIW*RSIN1)
		ANCXE2=D*FRPIW*RCOS1
		DO 60 IP=1,2
		  RCXA=RCXE2*RCXR2(IP)
		  ANG12=ANCXR2(IP)+ANCXE2
		  RCXB=1./(1.+RCXR1S(IP)*RCXA*RCXA+2.*RCXR1(IP)*RCXA*
     $           COS(ANG12+ANCXR1(IP)))
		  RCXA=RCXR1S(IP)+RCXA*RCXA+2.*RCXR1(IP)*RCXA*
     $           COS(ANG12-ANCXR1(IP))
		  RB=RCXA*RCXB
		  TB=RCXT1S(IP)*RCXT2S(IP)*RCXE2*RCXB
C
C               TOTAL T
C
		  RAIP=RA(IP)
		  W1=RAIP*RB
		  W2=1.0/(1.0-W1)
		  RBA=RB+TB*TB*RAIP*W2
		  W1=RBA*RAIP
		  TT(IP)=TATC(IP)*TB*W2/(1.0-W1)
   60         CONTINUE
		TD=TD+AG(ID)*(POLS(I)*(TT(1)-TT(2))+TT(2))
   65        CONTINUE
	    END IF
C          END OF THICKNESS LOOP
C
	    T=T+AG(IFI)*TD
   70    CONTINUE
C
C        END OF ANGLE AND BASELINE-SAMPLE LOOPS
	  ODT=0.6020599-ALOG10(T)
	  IF (IT.LE.0)THEN
	    CAY(I)=(ODY(I)-ODT)/(FPDM0*W)
	  ELSE
	    DCAY=(ODY(I)-ODT)/(FPDM0*W)
	    DEV=DEV+ABS(DCAY)
	    CAY(I)=CAY(I)+DCAY
	  END IF
   80  CONTINUE
C
C      END OF WAVENUMBER LOOP
C
	 WRITE(IPRNT,1180)IT
	 IF(IWRTA.NE.0)THEN
	  WRITE(*,*)'DOING BASELINE CORRECTION'
C
C         CALCULATE THE POINTS NP1 AND NP2 SUCH THAT X(NP1)>=WN(IWN)>=X(NP1+1)
C         AND X(NP2) >= WN(IWN+1) >= X(NP2+1)
C
	  NP2=INT((XSP-WN(1))/DELX)+1
	  RKWN2=CAY(NP2) + (X(NP2)-WN(1))*(CAY(NP2+1)-CAY(NP2))/DELX
	  AWN2=FPDM0*WN(1)*RKWN2
	  DIF2=AWN2-AREF(1)*DM0
	  WRITE(IPRNT,1170)
	  WRITE(IPRNT,1160)WN(1),-DIF2
	  
	  DO 79 IWN=2,ABS(IWRTA)
	    NP1=NP2
	    NP2=INT((XSP-WN(IWN))/DELX)+1
C
C          CALCULATE VALUES OF KAPP (EXPERIMENTAL) AT WN(IWN-1), AND WN(IWN). 
C          K IS ASSUMED TO BE LINEAR OVER THE RANGE X(NP1) TO X(NP1+1) AND
C          X(NP2) TO X(NP2+1)
C
	    RKWN1=RKWN2
	    RKWN2=CAY(NP2)+(X(NP2)-WN(IWN))*(CAY(NP2+1)-CAY(NP2))
     $      /DELX
C
C          CALCULATE THE EXPERIMENTAL ABSORBANCE AT WN(IWN-1) AND WN(IWN)
C
	    AWN1=AWN2
	    AWN2=FPDM0*WN(IWN)*RKWN2
C
C          CALCULATE DIFFERENCE BETWEEN ABS (EXPERIMENTAL) AND ABS (CORRECT)
C          AT WN(IWN-1) AND WN(IWN)
C
	    DIF1=DIF2
	    DIF2=AWN2-AREF(IWN)*DM0
	    WRITE(IPRNT,1160)WN(IWN),-DIF2
C
C          ASSUMING THE CORRECTION IS LINEAR OVER THE COMPLETE RANGE OF THE
C          SPECTRUM, CORRECT CAY (AND ODY)
C
	    IF(IWN.EQ.2)NP1=0
	    IF(IWN.EQ.ABS(IWRTA))NP2=NP
	    DO 81 I=NP1+1,NP2
	      DIF=(DIF2-DIF1)*(X(I)-WN(IWN-1))/(WN(IWN)-WN(IWN-1)) + DIF1
	      CAY(I)=CAY(I) - DIF/(FPDM0*X(I))
	      ODY(I)=ODY(I) - DIF
   81      CONTINUE
   79    CONTINUE
C
C          WRITE THE CORRECTED ABSORBANCE SPECTRUM TO DISK IF IWETA>0
C
	    IF(DOSPC)THEN
	      CALL WRITESC(CAFILE,ODY,NP,XSP,XEP,1,0,RES,COMM,IER)
	    ELSE
	      CALL WRITEAS(CAFILE,ODY,NP,XSP,XEP)
	    END IF
	 END IF
	  DEV=DEV/NP
	  IF (IT.GT.1.AND.DEV.GT.DEVP)THEN
	    WRITE (IPRNT,1060)
	    GO TO 110
	  END IF
	  IF(IT.GT.0)WRITE (IPRNT,1040) DEV
	  DEVP=DEV
	IF(.NOT.NFIX)THEN
	  CALL GETTIM(IH,IM,IS,IS100)
	  WRITE(*,1140) 'DOING KK TRANSFORM',IM,IS,IS100
	  CALL KKTSD
	END IF
	CALL GETTIM(IH,IM,IS,IS100)
	WRITE(*,1140) 'ITERATION COMPLETE',IM,IS,IS100
	IF (IT.GT.0.AND.DEV.LT.0.00002) GO TO 110
   45 CONTINUE
C
C     OUTPUT SECTION
C
      WRITE(IPRNT,1070)
  110 CONTINUE
      IF(DOSPC)THEN
	CALL WRITESC(KFILE,CAY,NP,XSP,XEP,1,0,RES,COMM,IER)
	IF(.NOT.NFIX)CALL WRITESC(NFILE,EN,NP,XSP,XEP,1,0,RES,COMM,IER)
      ELSE
	CALL WRITEAS(KFILE,CAY,NP,XSP,XEP)
	IF(.NOT.NFIX)CALL WRITEAS(NFILE,EN,NP,XSP,XEP)
      END IF
      CALL GETTIM(IH,IM,IS,IS100)
      WRITE(*,1140)'END OF RUN: TIME IS',IM,IS,IS100
  120 CLOSE(IPRNT)
      IF(IRUN.LT.IRUNT)GO TO 1
      CLOSE(IRD)
      STOP
C
 1000 FORMAT ('NOT FTIR DATA? DELX=',F6.3)
 1010 FORMAT (5X,'RNJ46A   DETERMINATION OF N AND K ',
     $'FROM ABSORBANCE.'/,20X,'VERSION 08/92'//)
 1020 FORMAT (20A4)
 1030 FORMAT (//10X,'REFRACTIVE INDEX OF INCIDENCE MEDIUM  = 1.0000'/10X
     $ 'REFRACTIVE INDEX FOR BASELINE CALC.',3X,'=',F7.4/10X,'MAXIMUM ',
     $ 'ANGLE OF INCIDENCE',12X,'=',F5.2,3X,'DEGREES  '/10X,'MEAN PATH ',
     $ 'LENGTH OF SAMPLE FILM',7X,'=',F8.3,' MICROMETRES'/10X,'WEDGE ',
     $ 'PARAMETER',23X,'=',F7.4,' MICROMETRES'/10X,'WINDOW TYPE',27X,'=',
     $ I4//)
 1040 FORMAT (/10X,'MEAN DEV. OF K = ',F7.5/)
 1060 FORMAT (/10X,'THE CALCULATION HAS BEGUN TO DIVERGE')
 1070 FORMAT (//10X,'THE MAXIMUM NUMBER OF ITERATIONS WAS REACHED ')
 1080 FORMAT (/6X,'MODE 1. CALCULATED USING A SUBTRACTIVE KRAMERS - KRON
     $IG PROCEDURE.'//6X,'REFERENCE POINT ',F7.2,F7.4/)
 1090 FORMAT (/6X,'MODE 2. CALCULATED USING A STANDARD KRAMERS - KRONIG
     $PROCEDURE'//6X,'REFERENCE REFRACTIVE INDEX = ',F7.4/)
 1130 FORMAT (1X,20A4)
 1135 FORMAT (/6X,'IWRTA = 0 : EXP ABS NOT CORRECTED',
     $ ' TO IDEAL EXP ABS')
 1136 FORMAT (/6X,'IWRTA <> 0 : EXP ABS CORRECTED',
     $ ' TO IDEAL EXP ABS')
 1140 FORMAT(1X,A30,1X,I2.2,':',I2.2,'.',I2.2)
 1150 FORMAT(/////,23X,'RNJ46A'/,2X,'OPTICAL CONSTANTS FROM ABSORBANCE',
     $ ' MEASUREMENTS'/,20X,'VERSION 08/92'//)
 1160 FORMAT(4X,F6.1,8X,F6.4)
 1170 FORMAT(2X,'Wavenumber',4x,'Correction')
 1180 FORMAT(2X, 'ITERATION ',I2)
 1190 FORMAT (/6X,'MODE 0. NO KK TRANSFORM, N TAKEN FROM ',A12/)
 1200 FORMAT(/2X,'Spacings in absorbance and n files are different. ',
     $ 'Correct and rerun.')
 1210 FORMAT(/2X,'N file is too small, correct and rerun.')
 1220 FORMAT (//10X,'REFRACTIVE INDEX OF INCIDENCE MEDIUM  = 1.0000'/10X
     $ ,'MAXIMUM ',
     $ 'ANGLE OF INCIDENCE',12X,'=',F5.2,3X,'DEGREES  '/10X,'MEAN PATH ',
     $ 'LENGTH OF SAMPLE FILM',7X,'=',F8.3,' MICROMETRES'/10X,'WEDGE ',
     $ 'PARAMETER',23X,'=',F7.4,' MICROMETRES'/10X,'WINDOW TYPE',27X,'=',
     $ I4//)
 
      END
